@@ -4,7 +4,8 @@
 #'   or endyear of the decennial Census or ACS sample. 5-year ACS data is
 #'   available from 2009 through 2018. 1-year ACS data is available from 2005
 #'   through 2019.
-#' @param dataset One of "sf1", "sf3", "acs1", "acs3", "acs5", "acs1/profile",
+#' @param dataset One of "sf1", "sf2", "sf3", "sf4", "pl",
+#'   "as", "gu", "mp", "vi", "acs1", "acs3", "acs5", "acs1/profile",
 #'   "acs3/profile, "acs5/profile", "acs1/subject", "acs3/subject", or
 #'   "acs5/subject".
 #' @param cache Whether you would like to cache the dataset for future access,
@@ -19,8 +20,12 @@
 #'
 load_variables <- function(year, dataset, cache = FALSE) {
 
+  if (year == 2020 && stringr::str_detect(dataset, "acs1")) {
+    stop("The 2020 1-year ACS was released as a set of experimental estimates that was not published to the Census API and is in turn not available in tidycensus.", call. = FALSE)
+  }
+
   if (year == 1990) {
-    stop("The 1990 decennial Census endpoint has been removed by the Census Bureau. We will support 1990 data again when the endpoint is updated; in the meantime, we recommend using NHGIS (https://nhgis.org) and the ipumsr R package.")
+    stop("The 1990 decennial Census endpoint has been removed by the Census Bureau. We will support 1990 data again when the endpoint is updated; in the meantime, we recommend using NHGIS (https://nhgis.org) and the ipumsr R package.", call. = FALSE)
   }
 
   if (dataset == "sf3" && year > 2001) {
@@ -41,14 +46,14 @@ load_variables <- function(year, dataset, cache = FALSE) {
 
   rds <- paste0(dataset, "_", year, ".rds")
 
-  if (grepl("^acs[135]/(profile|subject)$", dataset)) {
+  if (grepl("^acs[135]/(profile|subject|cprofile)$", dataset)) {
     rds <- gsub("/", "_", rds)
   }
 
   var_type <- NULL
 
-  if (str_detect(dataset, "/")) {
-    split <- str_split(dataset, "/")[[1]]
+  if (stringr::str_detect(dataset, "/")) {
+    split <- stringr::str_split(dataset, "/")[[1]]
     dataset <- split[1]
     var_type <- split[2]
   }
@@ -74,26 +79,40 @@ load_variables <- function(year, dataset, cache = FALSE) {
                  set,
                  "variables.json", sep = "/")
 
-    dat <- GET(url) %>%
-      content(as = "text") %>%
-      fromJSON() %>%
-      modify_depth(2, function(x) {
+    dat <- httr::GET(url) %>%
+      httr::content(as = "text") %>%
+      jsonlite::fromJSON() %>%
+      purrr::modify_depth(2, function(x) {
         x$validValues <- NULL
         x
       }) %>%
-      flatten_df(.id = "name") %>%
-      arrange(name)
+      purrr::flatten_df(.id = "name") %>%
+      dplyr::arrange(name)
 
     out <- dat[,1:3]
 
     names(out) <- tolower(names(out))
 
-    out1 <- out[grepl("^B[0-9]|^C[0-9]|^DP[0-9]|^S[0-9]|^P.*[0-9]|^H.*[0-9]|^K[0-9]", out$name), ]
+    out1 <- out[grepl("^B[0-9]|^C[0-9]|^DP[0-9]|^S[0-9]|^P.*[0-9]|^H.*[0-9]|^K[0-9]|^CP[0-9]",
+                      out$name), ]
 
-    out1$name <- str_replace(out1$name, "E$|M$", "")
+    out1$name <- stringr::str_replace(out1$name, "E$|M$", "")
 
     out2 <- out1[!grepl("Margin Of Error|Margin of Error", out1$label), ]
 
+    # Add geography information for acs5
+    if (dataset == "acs/acs5" && year > 2010) {
+
+      geo <- tidycensus::acs5_geography
+
+      geo_lookup <- geo[geo$year == year,]
+
+      out2 <- out2 %>%
+        dplyr::mutate(table = stringr::str_remove(name, "_.*")) %>%
+        dplyr::left_join(geo_lookup, by = "table") %>%
+        dplyr::select(-year, -table)
+    }
+
     return(as_tibble(out2))
   }
 
@@ -109,8 +128,17 @@ load_variables <- function(year, dataset, cache = FALSE) {
 
         out <- read_rds(file_loc)
 
+        # For 5-year ACS without geography, refresh the table
+        if (year > 2010 && dataset == "acs/acs5") {
+          if (!"geography" %in% names(out)) {
+            df <- get_dataset(dataset, year)
+            readr::write_rds(df, file_loc)
+            return(df)
+          }
+        }
+
         # For 2010 decennial Census, must get again if old
-        if (year == 2010 && dataset == "sf1") {
+        if (year == 2010 && dataset == "dec/sf1") {
 
           # Check if an erroring variable is in the file
           if ("H00010001" %in% out$name) {
@@ -127,7 +155,8 @@ load_variables <- function(year, dataset, cache = FALSE) {
           }
         }
 
-        out1 <- out[grepl("^B[0-9]|^C[0-9]|^DP[0-9]|^S[0-9]|^P.*[0-9]|^H.*[0-9]|^K[0-9]", out$name), ]
+        out1 <- out[grepl("^B[0-9]|^C[0-9]|^DP[0-9]|^S[0-9]|^P.*[0-9]|^H.*[0-9]|^K[0-9]|^CP[0-9]",
+                          out$name), ]
 
         out1$name <- str_replace(out1$name, "E$|M$", "")
 
